//: ### Floating Views
import PlaygroundSupport
import UIKit

class Decoration: UICollectionReusableView {
    override func apply(_ layoutAttributes: UICollectionViewLayoutAttributes) {
        super.apply(layoutAttributes)
        self.backgroundColor = .blue
        self.layer.borderWidth = 2
        self.layer.borderColor = UIColor.black.cgColor
    }
}

//: This is a subclass of the layout from the Custom Layout page
class FloatingDecoratedBoringLayout: BoringLayout {
    
    let decorationKind = "decoration"
    let decorationSize = CGSize(width: 20, height: 20)
    
    override init() {
        super.init()
        self.register(Decoration.self, forDecorationViewOfKind: decorationKind)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func layoutAttributesForElements(in rect: CGRect) -> [UICollectionViewLayoutAttributes]? {
        var attributes = super.layoutAttributesForElements(in: rect)
        // There is a single decoration view
        if let decorationAttributes = self.layoutAttributesForDecorationView(ofKind: decorationKind, at: IndexPath(item: 0, section: 0)) {
            attributes?.append(decorationAttributes)
        }
        return attributes
    }
    
    override func layoutAttributesForDecorationView(ofKind elementKind: String, at indexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
        print(#function)
        guard let collectionView = self.collectionView else { return nil }
        
        let attributes = UICollectionViewLayoutAttributes(forDecorationViewOfKind: elementKind, with: indexPath)
        // Calculate the frame each time
        let origin = CGPoint(
            x: collectionView.bounds.width - 20 - self.decorationSize.width,
            y: collectionView.bounds.origin.y + 20)
        attributes.frame = CGRect(origin: origin, size: self.decorationSize)
        // Setting this means the decoration view will always be above the cells, otherwise it becomes dependent on the order views were added, which is no good.
        attributes.zIndex = 10
        return attributes
    }
    
    // Returning true here means that the layout will then be asked for an invalidation context
    override func shouldInvalidateLayout(forBoundsChange newBounds: CGRect) -> Bool {
        print(#function)
        return true
    }
    
    override func invalidationContext(forBoundsChange newBounds: CGRect) -> UICollectionViewLayoutInvalidationContext {
        print(#function)
        let context = super.invalidationContext(forBoundsChange: newBounds)
        
        guard let collectionView = self.collectionView else {
            return context
        }
        
        if collectionView.bounds.size != newBounds.size {
            return context
        }
        
        // The size is the same, but it's a bounds change - this means we're scrolling
        context.invalidateDecorationElements(ofKind: decorationKind, at: [IndexPath(item: 0, section: 0)])
        return context
    }
    
}

let layout = FloatingDecoratedBoringLayout()

let collectionView = UICollectionView(frame: CGRect(x: 0, y: 0, width: 375, height: 667), collectionViewLayout: layout)
collectionView.register(TextCell.self, forCellWithReuseIdentifier: "cell")

let datasource = DataSource()
collectionView.dataSource = datasource
PlaygroundPage.current.liveView = collectionView

//: [Next](@next)
//:
//: [Previous](@previous)
